//===--- ModularizeHeaderFile.cpp - Callbacks for Preprocessor actions ---*- C++ -*-===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

#include "ModularizeHeaderFile.h"
#include "ModularizeUtilities.h"
#include "llvm/Support/raw_ostream.h"
#include "clang/Basic/SourceManager.h"

namespace Modularize {
using namespace llvm;
using namespace clang;

ModularizeHeaderFile::ModularizeHeaderFile(std::string name, Preprocessor& pp) :
  Name(name) {
}

ModularizeHeaderFile::~ModularizeHeaderFile() {
}

// Get highest line-numbered directive.
unsigned ModularizeHeaderFile::getLastLineNumber() {
  if (Directives.size() == 0)
    return 0;
  return Directives[Directives.size() - 1].getLineNumber();
}

// Return true if this snapshot matches the other snapshot.
bool ModularizeHeaderFile::match(ModularizeHeaderFile *other) {
  bool returnValue = true;
  // Walk conditional directives.  If the condition
  // (after macro substitutions) is not the same,
  // return false, meaning the header file snapshot doesn't match.
	ModularizePPDirectiveVectorIterator thisIter = getDirectivesBegin();
	ModularizePPDirectiveVectorIterator thisEnd = getDirectivesEnd();
	ModularizePPDirectiveVectorIterator otherIter = other->getDirectivesBegin();
	ModularizePPDirectiveVectorIterator otherEnd = other->getDirectivesEnd();
  for (; (thisIter != thisEnd) && (otherIter != otherEnd);
      ++thisIter, ++ otherIter) {
    if (!(*thisIter).match(&(*otherIter))) {
      returnValue = false;
      break;
    }
  }
  return returnValue;
}

// Display the information for debugging.
void ModularizeHeaderFile::dump(int level) {
  errs() << getIndent(level) << "preprocessor directives:\n";
	ModularizePPDirectiveVectorIterator dIter = getDirectivesBegin();
	ModularizePPDirectiveVectorIterator dEnd = getDirectivesEnd();
  for (; dIter != dEnd; ++dIter) {
    (*dIter).dump(level + 1, Name.c_str());
  }
}

}
