//===- ModularizeHeaderInstance.h - Stores header instance info-*- C++ -*-===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
///
/// \file
/// \brief Defines a class to store information about a set of instances of a
///   header being used where the conditional directives are equivalent.
///
//===----------------------------------------------------------------------===//

#ifndef MODULARIZEHEADERINSTANCE_H
#define MODULARIZEHEADERINSTANCE_H

#include "ModularizeHeaderFile.h"

namespace Modularize {
  using namespace clang;
  using namespace llvm;

typedef std::vector<std::string> ModularizeTopHeaderVector;
typedef ModularizeTopHeaderVector::iterator ModularizeTopHeaderVectorIterator;

/// \brief This class stores information about a set of instances of a
///   header being used where the conditional directives are equivalent.
class ModularizeHeaderInstance {
public:
  ModularizeHeaderInstance(ModularizeHeaderFile *HeaderFile, std::string topHeader);
  ~ModularizeHeaderInstance();

  // Header file accessors.
  ModularizeHeaderFile *getHeaderFile() { return HeaderFile; }
  void setHeaderFile(ModularizeHeaderFile *headerFile) { HeaderFile = headerFile; };

  // Top headers (the files directly passed to modularize) accessors.
  ModularizeTopHeaderVector &getTopHeaders() { return TopHeaders; }
  ModularizeTopHeaderVectorIterator getTopHeadersBegin() { return TopHeaders.begin(); }
  ModularizeTopHeaderVectorIterator getTopHeadersEnd() { return TopHeaders.end(); }
  bool hasTopHeader(std::string topHeader);
  void addTopHeader(std::string topHeader) { TopHeaders.push_back(topHeader); };

  // Display the information for debugging.
  void dump(int level = 0);
private:
    // This points to the object containing a unique snapshot of the
	// representations of the header preprocessor directives.
  ModularizeHeaderFile *HeaderFile;
    // This vector contains the names of the top-level headers that
    // contained the above unique snapshot.
  ModularizeTopHeaderVector TopHeaders;
};

// Collections for users of this typs.
typedef std::vector<ModularizeHeaderInstance *> ModularizeHeaderInstanceVector;
typedef ModularizeHeaderInstanceVector::iterator ModularizeHeaderInstanceVectorIterator;

}  // end namespace clang

#endif
