//===- ModularizeHeaderTracker.h - Represents header instances -*- C++ -*-===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
///
/// \file
/// \brief Defines a class to store header instance snapshot information
///   for one header file.
///
//===----------------------------------------------------------------------===//

#ifndef MODULARIZEHEADERTRACKER_H
#define MODULARIZEHEADERTRACKER_H

#include "ModularizeHeaderInstance.h"

namespace Modularize {
  using namespace clang;
  using namespace llvm;

/// \brief This class stores the header snapshot instances for a particular
///   header.
class ModularizeHeaderTracker {
public:
  ModularizeHeaderTracker(std::string& name, ModularizeHeaderFile *headerFile,
	  std::string topHeader);
  ~ModularizeHeaderTracker();

  // Header name accessors.
  std::string& getName() { return Name; }
  void setName(std::string name) { Name = name; };

  // Process a header snapshot.
  // If the snapshot differs, add a new header instance.
  void addHeaderFile(ModularizeHeaderFile *headerFile, std::string topHeader);
  // Add a header instance.
  void addHeaderInstance(ModularizeHeaderInstance *headerInstance);
  // Get instance count.  > 1 means there were mismatches.
  int getHeaderInstanceCount() { return HeaderInstances.size(); }
  // Get header instance vector.
  ModularizeHeaderInstanceVector &getHeaderInstances() { return HeaderInstances; }
  ModularizeHeaderInstanceVectorIterator getHeaderInstancesBegin() { return HeaderInstances.begin(); }
  ModularizeHeaderInstanceVectorIterator getHeaderInstancesEnd() { return HeaderInstances.end(); }
  ModularizeHeaderInstance *getHeaderInstance(int index) { return HeaderInstances[index]; }

  // Report instances where header snapshots differ.
  // Returns true if no errors, i.e. no differing snapshots.
  bool report();
  // Display the information for debugging.
  void dump(int level = 0);

private:
  std::string Name;
  ModularizeHeaderInstanceVector HeaderInstances;
};

// Collections for users of this typs.
typedef StringMap<ModularizeHeaderTracker *> ModularizeHeaderTrackerMap;
typedef ModularizeHeaderTrackerMap::iterator ModularizeHeaderTrackerMapIterator;

}  // end namespace clang

#endif
