//==- ModularizeMasterHeaderTracker.cpp - Stores header trackers -*- C++ -*-==//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

#include "ModularizeMasterHeaderTracker.h"
#include "ModularizeUtilities.h"
#include "llvm/Support/raw_ostream.h"
#include "clang/Basic/SourceManager.h"

namespace Modularize {
using namespace llvm;
using namespace clang;

ModularizeMasterHeaderTracker::ModularizeMasterHeaderTracker(bool disabled)
    : Disabled(disabled) {
}

ModularizeMasterHeaderTracker::~ModularizeMasterHeaderTracker() {
}

// Find an existing header tracker.
// Returns NULL if not found..
ModularizeHeaderTracker *ModularizeMasterHeaderTracker::getHeaderTracker(
	std::string headerName) {
  ModularizeHeaderTrackerMap::iterator iter = HeaderTrackers.find(headerName);
  if (iter != HeaderTrackers.end())
    return iter->second;
  return NULL;
}

// Process a header snapshot.
// If the header has be seen before, and the snapshot differs, add a new
// header instance.
void ModularizeMasterHeaderTracker::addHeaderFile(
	ModularizeHeaderFile *headerFile, std::string topHeader) {
  ModularizeHeaderTracker *headerTracker = getHeaderTracker(
	  headerFile->getName());
  if (headerTracker != NULL)
    headerTracker->addHeaderFile(headerFile, topHeader);
  else {
    headerTracker = new ModularizeHeaderTracker(
		headerFile->getName(), headerFile, topHeader);
    HeaderTrackers[headerTracker->getName()] = headerTracker;
  }
}

// Report instances where header snapshots differ.
// Returns true if no errors, i.e. no differing snapshots.
bool ModularizeMasterHeaderTracker::report() {
  if (Disabled)
    return true;
  bool returnValue = true;
  ModularizeHeaderTrackerMapIterator iter = HeaderTrackers.begin();
  ModularizeHeaderTrackerMapIterator end = HeaderTrackers.end();
  for (; iter != end; ++iter) {
    ModularizeHeaderTracker *headerTracker = (*iter).second;
    if (!headerTracker->report())
      returnValue = false;
  }
  return returnValue;
}

// Display the information for debugging.
void ModularizeMasterHeaderTracker::dump(int level) {
  errs() << getIndent(level) << "The headers:\n";
  ModularizeHeaderTrackerMapIterator iter = HeaderTrackers.begin();
  ModularizeHeaderTrackerMapIterator end = HeaderTrackers.end();
  for (; iter != end; ++iter) {
    ModularizeHeaderTracker *headerTracker = (*iter).second;
    headerTracker->dump(level + 1);
  }
}

}
