//===- ModularizeMasterHeaderTracker.h - Stores header trackers -*- C++ -*-===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
///
/// \file
/// \brief Defines a class to store all the header trackers, for use in finding
/// header instance differences.
///
//===----------------------------------------------------------------------===//

#ifndef MODULARIZEMASTERHEADERTRACKER_H
#define MODULARIZEMASTERHEADERTRACKER_H

#include "ModularizeHeaderFile.h"
#include "ModularizeHeaderTracker.h"

namespace Modularize {
  using namespace clang;
  using namespace llvm;

/// \brief This class stores all the header trackers, for use in finding
///   header instance differences.
class ModularizeMasterHeaderTracker {
public:
  ModularizeMasterHeaderTracker(bool disabled);
  ~ModularizeMasterHeaderTracker();

  // Return true if header preprocessing checking is disabled.
  bool isDisabled() { return Disabled; }

  // Return the map of all the header trackers.
  ModularizeHeaderTrackerMap& getHeaderTrackers() { return HeaderTrackers; }
  // Find an existing header tracker.
  // Returns NULL if not found..
  ModularizeHeaderTracker *getHeaderTracker(std::string headerName);
  // Process a header snapshot.
  // If the header has be seen before, and the snapshot differs, add a new
  // header instance.
  void addHeaderFile(ModularizeHeaderFile *headerFile, std::string topHeader);

  // Report instances where header snapshots differ.
  // Returns true if no errors, i.e. no differing snapshots.
  bool report();
  // Display the information for debugging.
  void dump(int level = 0);
private:
  // Set to true if header preprocessing checking is disabled.
  bool Disabled;
  // This map contains an entry for each header file encountered.
  ModularizeHeaderTrackerMap HeaderTrackers;
};

}  // end namespace clang

#endif
