//===--- ModularizePPCallbacks.h - Callbacks for Modularize PP -*- C++ --*-===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
///
/// \file
/// \brief Implements the PPCallbacks interface for modularize.
///
//===----------------------------------------------------------------------===//

#ifndef MODULARIZEPPCALLBACKS_H
#define MODULARIZEPPCALLBACKS_H

#include "clang/Lex/Preprocessor.h"
#include "clang/Basic/SourceLocation.h"
#include "ModularizeHeaderFile.h"
#include "ModularizeMasterHeaderTracker.h"

namespace Modularize {
  using namespace clang;
  using namespace llvm;

typedef StringMap<std::string> ModularizeSymbolMap;
typedef ModularizeSymbolMap::iterator ModularizeSymbolMapIterator;

/// \brief This class provides callback implementations to grab information
///   about preprocessor directives encountered.
class ModularizePPCallbacks : public PPCallbacks {
public:
  ModularizePPCallbacks(Preprocessor &pp, StringRef topFile,
	  ModularizeMasterHeaderTracker &masterHeaderTracker);
  ~ModularizePPCallbacks();

  // Overidden handlers.
  void FileChanged(SourceLocation Loc, FileChangeReason Reason,
                           SrcMgr::CharacteristicKind FileType,
                           FileID PrevFID = FileID());
  void EndOfMainFile();
  void MacroExpands(const Token &MacroNameTok, const MacroDirective *MD,
                            SourceRange Range, const MacroArgs *Args);
  void If(SourceLocation Loc, SourceRange ConditionRange);
  void Elif(SourceLocation Loc, SourceRange ConditionRange,
                    SourceLocation IfLoc);
  void Ifdef(SourceLocation Loc, const Token &MacroNameTok,
                     const MacroDirective *MD);
  void Ifndef(SourceLocation Loc, const Token &MacroNameTok,
                      const MacroDirective *MD);
private:
  // Local accessors.
  ModularizeHeaderFile *getHeaderFile(const std::string& name);
  void addHeaderFile(ModularizeHeaderFile *headerFile);

  // Get symbol map.
  ModularizeSymbolMap &getSymbols() { return Symbols; }
  // Add a symbol definition.  If the symbol already exists,
  // its value is replaced.
  void addSymbol(std::string name, std::string value);
  // Get symbol value.  Return empty string if not defined.
  std::string getSymbol(std::string name);
  // Get symbol value.  Return true if found.
  bool getSymbol(std::string name, std::string &value);

  // Local utilities.
  std::string getSourceLocationString(SourceLocation Loc);
  std::string getSourceLocationFile(SourceLocation Loc);
  unsigned getSourceLocationLineNumber(SourceLocation Loc);
  // Retrieve source snippet from file image.
  std::string getSourceSnippet(SourceRange sourceRange);
  // Do macro substitutions in a string.
  std::string getMacroSubstitutedString(std::string str);

private:
  // Local data.
  ModularizeMasterHeaderTracker &MasterHeaderTracker;
  std::string TopFile;
  Preprocessor &PP;
  ModularizeHeaderFile *RootHeaderFile;
  ModularizeHeaderFile *CurrentHeaderFile;
  ModularizeHeaderFileMap HeaderFileMap;
  ModularizeSymbolMap Symbols;

};

}  // end namespace clang

#endif
